----------------------------------------------------------------------------------------------------
-- TextUtils.lua
--
-- Text formatting commands
----------------------------------------------------------------------------------------------------

----------------------------------------------------------------------------------------------------
-- Utility functions
----------------------------------------------------------------------------------------------------

-- Given a 6-character string, convert to RGB values
function textutils_strToRgb(str)

	-- Check length
	local rgb = {}
	rgb[0] = 0
	rgb[1] = 0
	rgb[2] = 0
	if str:len() ~= 6 then
		return rgb
	end

	-- Loop for each colour component
	for i=0,2 do
		local s = str:sub(i*2+1, i*2+2):lower()

		local val = 0
		local mul = 16
		for j=1,2 do
			local sByte = s:byte(j, j)
			if sByte >= string.byte("0", 1, 1) and sByte <= string.byte("9", 1, 1) then
				sByte = sByte - string.byte("0", 1, 1)
			elseif sByte >= string.byte("a", 1, 1) and sByte <= string.byte("f", 1, 1) then
				sByte = sByte - string.byte("a", 1, 1) + 10
			else
				sByte = 0
			end

			val = val + sByte * mul
			mul = mul - 15
		end

		rgb[i] = val
	end

	return rgb
end

-- Returns the distance to skip in a string if it's a TA2 tag, or 0 otherwise
function getTagLen(str)
	local ret = "";
	local len = str:len();
	if str:sub(1,1) == "<" and str:sub(2,2) == "-" then

		-- Single character code?
		ch2 = str:sub(3,3):lower();
		ch3 = str:sub(4,4):lower();
		if str:sub(4,4) == ">" and (ch2 == "1" or ch2 == "0" or ch2 == "b" or ch2 == "i" or ch2 == "u" or ch2 == "s") then
			return 4;

		-- Telnet colour? 49 = "1":byte(1,1), 55 = "7":byte(1,1)
		elseif ch2 == "3" and str:sub(5,5) == ">" and str:byte(4,4) >= 49 and str:byte(4,4) <= 55 then
			return 5;

		-- Close code?
		elseif str:sub(3,3) == "/" and str:sub(5,5) == ">" and (ch3 == "b" or ch3 == "i" or ch3 == "u" or ch3 == "s") then
			return 5;

		-- 24-bit colour code? Don't bother asserting it's a valid colour...
		elseif ch2 == "c" and str:sub(10,10) == ">" then
			return 10;

		-- Other char
		else
			return 0;
		end
	else
		return 0;
	end

	return 0;
end

-- Strip user tags (E.g. <-31>, <-b>, <-cff9900>, <-/s>, etc) from a string
function strip_usertags(str)
	local ret = "";
	local len = str:len();
	local i=1;
	while i <= len do
		local tagLen = getTagLen(str:sub(i));
		if tagLen == 0 then
			ret = ret .. str:sub(i,i);
			i = i+1;
		else
			i = i+tagLen;
		end
	end

	return ret;
end

----------------------------------------------------------------------------------------------------
-- Blend text betweeen two colours
----------------------------------------------------------------------------------------------------

function textutils_blend(str)

	-- Extract from and to colours
	if str:len() < 15 or str:sub(7,7) ~= " " or str:sub(14,14) ~= " " then
		println("Usage: /blend rrggbb rrggbb string")
		return
	end
	local from = textutils_strToRgb(str:sub(1,6))
	local to = textutils_strToRgb(str:sub(8,13))
	str = str:sub(15)

	-- Clamp RGB, since TA will do it if we don't
	if from[0] < 0x40 and from[1] < 0x40 and from[2] < 0xa4 then
		for i=0,2 do
			if from[i] < 0x40 then
				from[i] = 0x40
			end
		end
	end
	if to[0] < 0x40 and to[1] < 0x40 and to[2] < 0xa4 then
		for i=0,2 do
			if to[i] < 0x40 then
				to[i] = 0x40
			end
		end
	end

	-- Format
	local ret = ""
	local strippedText = strip_usertags(str);
	local strippedLen = strippedText:len();
	local i=1;
	local strippedPos=1;
	while i <= str:len() do
		local tagLen = getTagLen(str:sub(i));
		if tagLen == 0 then
			local t = (strippedPos-1) / (strippedLen - 1)
			local r = from[0] * (1-t) + to[0] * t
			local g = from[1] * (1-t) + to[1] * t
			local b = from[2] * (1-t) + to[2] * t
			ret = ret .. string.format("<-c%02x%02x%02x>%s", r, g, b, str:sub(i,i))
			i = i+1;
			strippedPos = strippedPos+1;
		else
			ret = ret .. str:sub(i,i+tagLen-1);
			i = i+tagLen;
		end
	end
	sendln(ret)
end

register_command("blend", "textutils_blend")
